function initialdata!(env, P::Project, prms)
  initialdata_equation!(env, env.mesh, P, P.equation, prms)
  return
  initialdata_hrsc!(env, P, P.hrsc, prms)
  initialdata_tci!(env, P, P.tci, prms)
end


function initialdata_equation!(env, mesh::Mesh1d, P::Project, equation::AbstractScalarEq, prms)

  @unpack cache, mesh = env
  @unpack u = get_dynamic_variables(cache)
  @unpack L, x, xmin, xmax = mesh

  @unpack id = prms
  x = x[:]

  id_fn = if id == "sine"
    @unpack id_sine_period, id_sine_phase = prms
    u .= @. sin.(2 * pi * id_sine_period * x/L + id_sine_phase)
  elseif id == "bump"
    @unpack id_bump_height, id_bump_width = prms
    @toggled_assert id_bump_width < L "Initialdata 'bump': Bump width exceeds x domain '$xrange'"
    xl, xr = xmin+(L-id_bump_width)/2, xmin+(L+id_bump_width)/2
    u .= @. (x < xl || x > xr) ? 0.5 : id_bump_height
  elseif id == "gaussian"
    A = prms["id_gaussian_amplitude"]
    w = prms["id_gaussian_width"]
    @toggled_assert w > 0.0
    x0 = (xmax + xmin) / 2
    u .= @. A * exp(-(x-x0)^2 / w^2)
  elseif id == "smooth bump"
    error("TODO Implement 'smooth bump' initial data")
#   elseif type === :smooth_bump
#     xl = xmin + L * 0.10
#     xr = xmin + L * 0.90
#     gap_x = 0.3 * L
#     u0 = zeros(size(x))
#     for (i, xi) in enumerate(x)
#       if xi < xl || xi > xr
#         u0[i] = 0.0
#       elseif xi > xl+gap_x && xi < xr-gap_x
#         u0[i] = 1.0
#       elseif xi >= xl && xi <= xl+gap_x
#         u0[i] = sin( (xi - xl) / gap_x * 0.5 * π)^2
#       elseif xi >= xr-gap_x && xi <= xr
#         u0[i] = 1.0 - sin( (xi - (xr - gap_x)) / gap_x * 0.5 * π)^2
#       end
#     end
  else
    error("Unknown initialdata requested: '$id'")
  end

  return
end


function initialdata_equation!(env, mesh::Mesh2d, P::Project, equation::AbstractScalarEq, prms)
  @unpack cache, mesh = env
  @unpack u = get_dynamic_variables(cache)
  @unpack x, y = get_static_variables(cache)
  @unpack id = prms

  (xmin, xmax), (ymin, ymax) = mesh.extends

  Lx = xmax - xmin
  Ly = ymax - ymin
  alpha = atan(Ly,Lx)
  LL = sqrt(Lx^2 + Ly^2)

  id_fn = if id == "sine"
    # @unpack id_sine_period, id_sine_phase = prms
    # u .= @. sin.(2 * pi * id_sine_period * x/L + id_sine_phase)
    @. u = sin(2.0 * pi * (x/Lx + y/Ly))
  elseif id == "gaussian"
    A = prms["id_gaussian_amplitude"]
    w = prms["id_gaussian_width"]
    @toggled_assert w > 0.0
    x0 = (xmax + xmin) / 2
    y0 = (ymax + ymin) / 2
    @. u .= A * exp(-((x-x0)^2 + (y-y0)^2) / w^2)
  end

  return
end


function initialdata_hrsc!(env, P::Project, hrsc::Maybe{HRSC.AbstractHRSC}, prms) end


function initialdata_hrsc!(env, P::Project, hrsc::HRSC.AbstractArtificialViscosity, prms)
  @unpack mu = get_cell_variables(env.cache)
  @. mu = 0.0
end


function initialdata_hrsc!(env, P::Project, hrsc::HRSC.EntropyViscosity, prms)
  @unpack E, Em1, F, Fm1 = get_static_variables(env.cache)
  @unpack mu             = get_cell_variables(env.cache)
  @. E = 0.0
  @. F = 0.0
  @. Em1 = 0.0
  @. Fm1 = 0.0
  @. mu = 0.0
end


function initialdata_tci!(env, P::Project, tci::Nothing, prms) end


function initialdata_tci!(env, P::Project, tci::TCI.AbstractTCI, prms)
  @unpack flag = get_cell_variables(env.cache)
  @. flag = 0.0
end
